typedef unsigned int size_t;
typedef unsigned int ptrdiff_t;

namespace std
{
   template <class T> class allocator;

   // specialize for void:
   template <>
   class allocator<void>
   {
   public:
      typedef void*       pointer;
      typedef const void* const_pointer;
      // reference-to-void members are impossible
      typedef void        value_type;

      template <class U>
      struct rebind
      {
         typedef allocator<U> other;
      };
   };

   template <class T>
   class allocator
   {
   public:
      typedef size_t    size_type;
      typedef ptrdiff_t difference_type;
      typedef T*        pointer;
      typedef const T*  const_pointer;
      typedef T&        reference;
      typedef const T&  const_reference;
      typedef T         value_type;

      template <class U>
      struct rebind
      {
         typedef allocator<U> other;
      };

      allocator() throw() {}
      allocator( const allocator& ) throw();
      template <class U>
      allocator( const allocator<U>& ) throw();
      ~allocator() throw() {}

      pointer address( reference x) const;
      const_pointer address( const_reference x) const;

      pointer allocate( size_type n, 
                        allocator<void>::const_pointer hint = 0 )
      {
        return pointer( ::operator new( n * sizeof(T) ) );
      }                        
      void deallocate( pointer p, size_type )
      {
        ::operator delete( p );
      }
      
      size_type max_size() const throw();

      void construct( pointer p, const T& val );
      void destroy( pointer p );
   };
}
